"""File Input/Output utilities."""

from download import download
import os.path as op
import os
import matplotlib.pyplot as plt

plt.style.use('ggplot')

# Data URLs, structured as {'week_name': [(URL, FILENAME, FILETYPE)]}
# If ZIPFILE / TARFILE / etc, we'll unzip to a folder w/ the week's name
DATA_URLS = {
    'week_02': [('https://ndownloader.figshare.com/files/7010681', 'boulder-precip.csv', 'file'),
                ('https://ndownloader.figshare.com/files/7010681', 'temperature_example.csv', 'file'),
                ('https://ndownloader.figshare.com/files/7426738', '.', 'zip')],
    'week_03': ('https://ndownloader.figshare.com/files/7446715', '.', 'zip'),
    'week_04': ('https://ndownloader.figshare.com/files/7525363', '.', 'zip'),
    'week_05': ('https://ndownloader.figshare.com/files/7525363', '.', 'zip'),
    'week_07': ('https://ndownloader.figshare.com/files/7677208', '.', 'zip'),
    'week_08': [('https://ndownloader.figshare.com/files/9666637?private_link=480fba92b3e882c4d35d', 'week_08', 'zip'),
                ('https://ndownloader.figshare.com/files/9666637?private_link=480fba92b3e882c4d35d', 'week_08-hw', 'tar')],
    # Just for testing - I have no idea what this file is but it's the 1st that popped up on google lol
    'test': [('https://www.colorado.edu/conflict/peace/download/peace_essay.ZIP', 'testfile', 'file'),
             ('https://www.colorado.edu/conflict/peace/download/peace_essay.ZIP', 'testfile_folder', 'zip'),
             ('https://www.colorado.edu/conflict/peace/download/peace_essay.ZIP', '.', 'zip')],
}

ALLOWED_FILE_TYPES = ['zip', 'tar', 'tar.gz', 'file']

#               destfile = "data/boulder-precip.csv"'}
HOME = op.join(op.expanduser('~'))
DATA_NAME = op.join('earth-analytics', 'data')


class EarthlabData(object):
    """
    Data storage and retrieval functionality for Earthlab.

    Parameters
    ----------
    path : string | None
        The path where data is stored.
    """
    def __init__(self, path=None):
        if path is None:
            path = op.join(HOME, DATA_NAME)
        self.path = path
        self.data_keys = list(DATA_URLS.keys())

    def __repr__(self):
        s = 'Available Datasets: {}'.format(self.data_keys)
        return s

    def get_data(self, key=None, name=None, replace=False):
        """
        Retrieve the data for a given week and return its path.

        This will retrieve data from the internet if it isn't already
        downloaded, otherwise it will only return a path to that dataset.

        Parameters
        ----------
        key : str
            The dataset to retrieve. Possible options can be found in
            ``self.data_keys``.
        replace : bool
            Whether to replace the data for this key if it is
            already downloaded.

        Returns
        -------
        path_data : str
            The path to the downloaded data.
        """
        if key is None:
            print('Available datasets: {}'.format(
                list(DATA_URLS.keys())))
        elif key not in DATA_URLS:
            raise ValueError("Don't understand key "
                             "{}\nChoose one of {}".format(
                                key, DATA_URLS.keys()))
        else:
            this_root = op.join(self.path, key)
            this_data = DATA_URLS[key]
            if not isinstance(this_data, list):
                this_data = [this_data]
            data_paths = []
            for url, name, kind in this_data:
                if kind not in ALLOWED_FILE_TYPES:
                    raise ValueError('kind must be one of {}, got {}'.format(ALLOWED_FILE_TYPES, kind))

                # If kind is not 'file' it will be un-archived to a folder w/ `name`
                # else create a file called `name`
                this_path = download(url, os.path.join(this_root, name),
                                     replace=replace, kind=kind,
                                     verbose=False)
                data_paths.append(this_path)
            if len(data_paths) == 1:
                data_paths = data_paths[0]
            return data_paths


def list_files(path, depth=3):
    """
    List files in a directory up to a specified depth.

    Parameters
    ----------
    path : str
        A path to a folder whose contents you want to list recursively.
    depth : int
        The depth of files / folders you want to list inside of ``path``.
    """
    if not os.path.isdir(path):
        raise ValueError('path: {} is not a directory'.format(path))
    depth_str_base = '  '
    if not path.endswith(os.sep):
        path = path + os.sep

    for ii, (i_path, folders, files) in enumerate(os.walk(path)):
        folder_name = op.basename(i_path)
        path_wo_base = i_path.replace(path, '')
        this_depth = len(path_wo_base.split('/'))
        if this_depth > depth:
            continue

        # Define the string for this level
        depth_str = depth_str_base * this_depth
        print(depth_str + folder_name)

        if this_depth + 1 > depth:
            continue
        for ifile in files:
            print(depth_str + depth_str_base + ifile)
